==> eudcc-0.0.1.tar.gz: lib/eudcc.dart <==
library eudcc;

import 'dart:convert';
import 'dart:io';
import 'dart:typed_data';

import 'package:eudcc/certificates/certificate.dart';
import 'package:eudcc/certificates/valuesets/country.dart';
import 'package:eudcc/certificates/valuesets/vaccine.dart' as vaccine;
import 'package:eudcc/certificates/valuesets/test.dart' as test;
import 'package:eudcc/certificates/valuesets/target.dart';
import 'package:eudcc/trust.dart';
import 'package:flutter/material.dart';
import 'package:typed_data/typed_data.dart' as typed;
import 'package:cbor/cbor.dart';
import 'base45.dart';

/*
import sys
import zlib
import pprint
# Modules tiers (Pillow, Pyzbar, base45, cbor2)
import PIL.Image
import pyzbar.pyzbar
import base45
import cbor2

img = PIL.Image.open("qrcode.png")
data = pyzbar.pyzbar.decode(img)
cert = data[0].data.decode()
b45data = cert.replace("HC1:", "")
zlibdata = base45.b45decode(b45data)
cbordata = zlib.decompress(zlibdata)
decoded = cbor2.loads(cbordata)
print("Header\n----------------");
pprint.pprint(cbor2.loads(decoded.value[0]))
print("\nPayload\n----------------");
pprint.pprint(cbor2.loads(decoded.value[2]))
print("\nSignature ?\n----------------");
print(decoded.value[3])
*/
class Parts {
  final typed.Uint8Buffer protectedHeader;
  final Map<dynamic, dynamic> unprotectedHeader;
  final typed.Uint8Buffer payload;
  final typed.Uint8Buffer signature;

  Parts(this.protectedHeader, this.unprotectedHeader, this.payload,
      this.signature);
}

enum CheckResult {
  valid,
  notCheck,
  invalid,
  unknownSigner,
  unknownAlgorithm,
  unknonwn,
}

class EUDigitalCovidCertificate {
  final Country issuer;
  final DateTime issuedAt;
  final DateTime expiresAt;
  final List<Certificate> certificates;
  final CheckResult check;

  EUDigitalCovidCertificate(
      {required this.issuer,
      required this.issuedAt,
      required this.expiresAt,
      required this.certificates,
      required this.check});
}

class EUDigitalCovidCertificateFactory {
  static final EUDigitalCovidCertificateFactory _singleton =
      EUDigitalCovidCertificateFactory._internal();

  factory EUDigitalCovidCertificateFactory() {
    return _singleton;
  }

  EUDigitalCovidCertificateFactory._internal();

  static const _header = "HC1";

  Parts _extract(String raw) {
    if (!raw.startsWith(_header)) {
      throw Exception("invalid header ${raw.substring(0, 5)}...");
    }
    raw = raw.substring(_header.length);
    if (raw.startsWith(":")) {
      raw = raw.substring(1);
    }
    var zdata = Base45.decode(raw);

    List<int> data;
    if (zdata[0] == 0x78) {
      data = gzip.decode(zdata);
    } else {
      throw Exception("unknown compression ${zdata[0]}");
    }

    // First level of CBOR
    var cbor = Cbor();
    cbor.decodeFromList(data);
    var dec = cbor.getDecodedData()![0];
    return Parts(dec[0], dec[1], dec[2], dec[3]);
  }

  CheckResult _check(Parts parts) {
    // Extract the header
    var c = Cbor();
    c.decodeFromBuffer(parts.protectedHeader);
    var pHeader = c.getDecodedData()![0];

    // Find signature
    var kid = base64Encode(
        pHeader.containsKey(4) ? pHeader[4] : parts.unprotectedHeader[4]);
    var alg = pHeader.containsKey(1) ? pHeader[1] : parts.unprotectedHeader[1];

    if (!TrustStore().containsKey(kid)) {
      return CheckResult.unknownSigner;
    }

    // Check!
    Trust t = TrustStore().get(kid)!;

    var data = Cbor();
    data.encoder.writeArray([
      'Signature1',
      Uint8List.view(
          parts.protectedHeader.buffer, 0, parts.protectedHeader.length),
      Uint8List(0),
      Uint8List.view(parts.payload.buffer, 0, parts.payload.length),
    ]);
    data.decodeFromInput();

    return t.verify(alg, data.output.getData(), parts.signature);
  }

  EUDigitalCovidCertificate? fromQrCode(String raw) {
    // Decode the QRCode
    var parts = _extract(raw);

    // Check for the signature
    var check = _check(parts);

    // Extract data and parse content
    var c = Cbor();
    c.decodeFromBuffer(parts.payload);
    var payload = c.getDecodedData()![0];

    // Extract content
    var issuer = payload[1];
    var issuedAt = DateTime.fromMicrosecondsSinceEpoch(payload[6] * 1000000);
    var expiresAt = DateTime.fromMicrosecondsSinceEpoch(payload[4] * 1000000);
    var certificates = payload[-260].values.toList();

    List<Certificate> certs = [];
    certificates.forEach((cert) {
      _mapConvert(Map<dynamic, dynamic> m) {
        Map<String, dynamic> json = {};
        m.forEach((key, value) {
          if (key is! String) {
            return;
          }
          if (value is Map<dynamic, dynamic>) {
            value = _mapConvert(value);
          }
          if (value is List<dynamic>) {
            value = value.map((v) {
              if (v is Map<dynamic, dynamic>) {
                v = _mapConvert(v);
              }
              return v;
            }).toList();
          }
          json[key] = value;
        });
        return json;
      }

      var j = _mapConvert(cert);
      certs.add(Certificate.fromJson(j));
    });

    return EUDigitalCovidCertificate(
      issuer: countryFromJson(issuer),
      issuedAt: issuedAt,
      expiresAt: expiresAt,
      certificates: certs,
      check: check,
    );
  }

  Future<void> loadDataset(BuildContext? context) async {
    await Future.wait([
      CountrySet().load(),
      TargetSet().load(),
      vaccine.ManufacturerSet().load(),
      vaccine.ProductSet().load(),
      vaccine.ProphylaxisSet().load(),
      test.ManufacturerSet().load(),
      test.ResultSet().load(),
      test.TypeSet().load(),
      TrustStore().load(),
    ]);
  }
}

==> eudcc-0.0.2.tar.gz: lib/eudcc.dart <==
library eudcc;

import 'dart:convert';
import 'dart:io';
import 'dart:typed_data';

import 'certificates/certificate.dart';
import 'certificates/valuesets/country.dart';
import 'certificates/valuesets/vaccine.dart' as vaccine;
import 'certificates/valuesets/test.dart' as test;
import 'certificates/valuesets/target.dart';
import 'trust.dart';
import 'package:flutter/material.dart';
import 'package:typed_data/typed_data.dart' as typed;
import 'package:cbor/cbor.dart';
import 'base45.dart';

/*
import sys
import zlib
import pprint
# Modules tiers (Pillow, Pyzbar, base45, cbor2)
import PIL.Image
import pyzbar.pyzbar
import base45
import cbor2

img = PIL.Image.open("qrcode.png")
data = pyzbar.pyzbar.decode(img)
cert = data[0].data.decode()
b45data = cert.replace("HC1:", "")
zlibdata = base45.b45decode(b45data)
cbordata = zlib.decompress(zlibdata)
decoded = cbor2.loads(cbordata)
print("Header\n----------------");
pprint.pprint(cbor2.loads(decoded.value[0]))
print("\nPayload\n----------------");
pprint.pprint(cbor2.loads(decoded.value[2]))
print("\nSignature ?\n----------------");
print(decoded.value[3])
*/
class Parts {
  final typed.Uint8Buffer protectedHeader;
  final Map<dynamic, dynamic> unprotectedHeader;
  final typed.Uint8Buffer payload;
  final typed.Uint8Buffer signature;

  Parts(this.protectedHeader, this.unprotectedHeader, this.payload,
      this.signature);
}

enum CheckResult {
  valid,
  notCheck,
  invalid,
  unknownSigner,
  unknownAlgorithm,
  unknonwn,
}

class EUDigitalCovidCertificate {
  final String? qr;
  final Country issuer;
  final DateTime issuedAt;
  final DateTime expiresAt;
  final List<Certificate> certificates;
  final CheckResult check;

  EUDigitalCovidCertificate(
      {required this.issuer,
      required this.issuedAt,
      required this.expiresAt,
      required this.certificates,
      required this.check,
      this.qr});
}

class EUDigitalCovidCertificateFactory {
  static final EUDigitalCovidCertificateFactory _singleton =
      EUDigitalCovidCertificateFactory._internal();

  factory EUDigitalCovidCertificateFactory() {
    return _singleton;
  }

  EUDigitalCovidCertificateFactory._internal();

  static const _header = "HC1";

  Parts _extract(String raw) {
    if (!raw.startsWith(_header)) {
      throw Exception("invalid header ${raw.substring(0, 5)}...");
    }
    raw = raw.substring(_header.length);
    if (raw.startsWith(":")) {
      raw = raw.substring(1);
    }
    var zdata = Base45.decode(raw);

    List<int> data;
    if (zdata[0] == 0x78) {
      data = gzip.decode(zdata);
    } else {
      throw Exception("unknown compression ${zdata[0]}");
    }

    // First level of CBOR
    var cbor = Cbor();
    cbor.decodeFromList(data);
    var dec = cbor.getDecodedData()![0];
    return Parts(dec[0], dec[1], dec[2], dec[3]);
  }

  CheckResult _check(Parts parts) {
    // Extract the header
    var c = Cbor();
    c.decodeFromBuffer(parts.protectedHeader);
    var pHeader = c.getDecodedData()![0];

    // Find signature
    var kid = base64Encode(
        pHeader.containsKey(4) ? pHeader[4] : parts.unprotectedHeader[4]);
    var alg = pHeader.containsKey(1) ? pHeader[1] : parts.unprotectedHeader[1];

    if (!TrustStore().containsKey(kid)) {
      return CheckResult.unknownSigner;
    }

    // Check!
    Trust t = TrustStore().get(kid)!;

    var data = Cbor();
    data.encoder.writeArray([
      'Signature1',
      Uint8List.view(
          parts.protectedHeader.buffer, 0, parts.protectedHeader.length),
      Uint8List(0),
      Uint8List.view(parts.payload.buffer, 0, parts.payload.length),
    ]);
    data.decodeFromInput();

    return t.verify(alg, data.output.getData(), parts.signature);
  }

  EUDigitalCovidCertificate? fromQrCode(String raw) {
    // Decode the QRCode
    var parts = _extract(raw);

    // Check for the signature
    var check = _check(parts);

    // Extract data and parse content
    var c = Cbor();
    c.decodeFromBuffer(parts.payload);
    var payload = c.getDecodedData()![0];

    // Extract content
    var issuer = payload[1];
    var issuedAt = DateTime.fromMicrosecondsSinceEpoch(payload[6] * 1000000);
    var expiresAt = DateTime.fromMicrosecondsSinceEpoch(payload[4] * 1000000);
    var certificates = payload[-260].values.toList();

    List<Certificate> certs = [];
    certificates.forEach((cert) {
      _mapConvert(Map<dynamic, dynamic> m) {
        Map<String, dynamic> json = {};
        m.forEach((key, value) {
          if (key is! String) {
            return;
          }
          if (value is Map<dynamic, dynamic>) {
            value = _mapConvert(value);
          }
          if (value is List<dynamic>) {
            value = value.map((v) {
              if (v is Map<dynamic, dynamic>) {
                v = _mapConvert(v);
              }
              return v;
            }).toList();
          }
          json[key] = value;
        });
        return json;
      }

      var j = _mapConvert(cert);
      certs.add(Certificate.fromJson(j));
    });

    return EUDigitalCovidCertificate(
      qr: raw,
      issuer: countryFromJson(issuer),
      issuedAt: issuedAt,
      expiresAt: expiresAt,
      certificates: certs,
      check: check,
    );
  }

  Future<void> loadDataset(BuildContext? context) async {
    await Future.wait([
      CountrySet().load(),
      TargetSet().load(),
      vaccine.ManufacturerSet().load(),
      vaccine.ProductSet().load(),
      vaccine.ProphylaxisSet().load(),
      test.ManufacturerSet().load(),
      test.ResultSet().load(),
      test.TypeSet().load(),
      TrustStore().load(),
    ]);
  }
}
